# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-05-03 11:26
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [("contenttypes", "0002_remove_content_type_name")]

    operations = [
        migrations.CreateModel(
            name="Index",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.TextField()),
                ("unique", models.BooleanField()),
                ("field", django.contrib.postgres.fields.jsonb.JSONField()),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.ContentType",
                    ),
                ),
            ],
        )
    ]



import django.contrib.postgres.fields as pg
from django.contrib.contenttypes.models import ContentType
from django.db import models


class Index(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    name = models.TextField()
    unique = models.BooleanField()
    field = pg.JSONField()
    where = pg.JSONField(null=True, blank=True)

    def get_structure(self):
        return {
            "name": self.name,
            "fields": set(self.field),
            "unique": self.unique,
            "where": set(self.where) if self.where else None,
        }



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2019-08-06 06:17
from __future__ import unicode_literals

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("django_indices", "0001_initial")]

    operations = [
        migrations.AddField(
            model_name="index",
            name="where",
            field=django.contrib.postgres.fields.jsonb.JSONField(blank=True, null=True),
        )
    ]



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-indices",
    version="0.0.3",
    author="Jatinderjit Singh",
    author_email="jatinderjit89@gmail.com",
    description="Create JSON indexes concurrently",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/jatinderjit/django-indices",
    packages=setuptools.find_packages(),
    classifiers=[
        "Framework :: Django",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from typing import List

from django.db import connections


class JSONIndex:
    def __init__(
        self, field: List[str], name: str, unique=False, concurrently=True, where=None
    ):
        self.field = field
        self.name = name
        self.concurrently = concurrently
        self.unique = unique
        self.where = where
        self.model = None

    def get_structure(self, model):
        if type(self.field[0]) is list or type(self.field[0]) is tuple:
            fields = set(self.get_field(f) for f in self.field)
        else:
            fields = {self.get_field(self.field)}

        where = self.where
        if where:
            query = model.objects.all().query
            compiler = query.get_compiler("default")
            where = where.resolve_expression(query).as_sql(
                compiler, connections["default"]
            )

        return {
            "name": self.name,
            "fields": fields,
            "unique": self.unique,
            "where": where,
        }

    def get_sql(self, model):
        concurrently = "CONCURRENTLY" if self.concurrently else ""
        structure = self.get_structure(model)
        unique = "UNIQUE" if structure["unique"] else ""
        fields = structure["fields"]
        if type(fields) is set:
            fields = ", ".join(fields)
            fields = f"({fields})"

        where = structure["where"]
        params = {}

        if where:
            where, params = where
            where = f"WHERE {where}"

        return (
            f"""
                CREATE {unique} INDEX {concurrently}
                IF NOT EXISTS {self.name}
                ON {model._meta.db_table} {fields}
                {where}
            """,
            params,
        )

    @staticmethod
    def get_field(field: List[str]):
        field = field[:]
        for i, part in enumerate(field[1:], 1):
            field[i] = f"'{part}'"
        field = "->".join(field[:-1]) + "->>" + field[-1]
        return f"({field})"
